{% extends "admin/base_site.html" %}
{% load i18n %}
{% load url from future %}
{% block userlinks %}{% url 'django-admindocs-docroot' as docsroot %}{% if docsroot %}<a href="{{ docsroot }}">{% trans 'Documentation' %}</a> / {% endif %}{% trans 'Change password' %} / <a href="../../logout/">{% trans 'Log out' %}</a>{% endblock %}
{% block breadcrumbs %}<div class="breadcrumbs"><a href="../../">{% trans 'Home' %}</a> &rsaquo; {% trans 'Password change' %}</div>{% endblock %}

{% block title %}{% trans 'Password change successful' %}{% endblock %}

{% block content %}

<h1>{% trans 'Password change successful' %}</h1>

<p>{% trans 'Your password was changed.' %}</p>

{% endblock %}
