{% extends "admin/base_site.html" %}
{% load i18n %}
{% block breadcrumbs %}<div class="breadcrumbs"><a href="../../../">Home</a> &rsaquo; <a href="../../">Documentation</a> &rsaquo; <a href="../">Views</a> &rsaquo; {{ name }}</div>{% endblock %}
{% block title %}View: {{ name }}{% endblock %}

{% block content %}

<h1>{{ name }}</h1>

<h2 class="subhead">{{ summary|striptags }}</h2>

{{ body }}

{% if meta.Context %}
<h3>Context:</h3>
<p>{{ meta.Context }}</p>
{% endif %}

{% if meta.Templates %}
<h3>Templates:</h3>
<p>{{ meta.Templates }}</p>
{% endif %}

<p class="small"><a href="../">&lsaquo; Back to Views Documentation</a></p>
{% endblock %}
